@@ -1,81 +1,89 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-import itertools
+import os
 import re
+from typing import Any, Dict, Iterable, List, Optional
+from urllib.parse import urljoin
 
-import jellyfish
+import jellyfish  # type: ignore
 
 FIPS_RE = re.compile(r"^\d{2}$")
 ABBR_RE = re.compile(r"^[a-zA-Z]{2}$")
 
-
-_lookup_cache = {}
+DC_STATEHOOD = bool(os.environ.get("DC_STATEHOOD"))
 
 
-class State(object):
+_lookup_cache: Dict[str, "State"] = {}
+
+
+class State:
+    abbr: str
+    ap_abbr: Optional[str]
+    capital: Optional[str]
+    capital_tz: Optional[str]
+    fips: Optional[str]
+    is_territory: bool
+    is_obsolete: bool
+    is_contiguous: bool
+    is_continental: bool
+    name: str
+    name_metaphone: str
+    statehood_year: Optional[int]
+    time_zones: List[str]
+
     def __init__(self, **kwargs):
         for k, v in kwargs.items():
-            self.__dict__[k] = v
+            setattr(self, k, v)
 
-    def __repr__(self):
-        return "<State:%s>" % self.name
+    def __repr__(self) -> str:
+        return f"<State:{self.name}>"
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
-    def shapefile_urls(self, region=None):
+    def shapefile_urls(self) -> Optional[Dict[str, str]]:
+        """Shapefiles are available directly from the US Census Bureau:
+        https://www.census.gov/cgi-bin/geo/shapefiles/index.php
+        """
 
-        if not self.fips:
-            return {}
+        fips = self.fips
 
-        base_url = "https://www2.census.gov/geo/tiger/TIGER2010"
+        if not fips:
+            return None
+
+        base = "https://www2.census.gov/geo/tiger/TIGER2010/"
         urls = {
-            "tract": "{0}/TRACT/2010/tl_2010_{1}_tract10.zip".format(
-                base_url, self.fips
-            ),
-            "cd": "{0}/CD/111/tl_2010_{1}_cd111.zip".format(base_url, self.fips),
-            "county": "{0}/COUNTY/2010/tl_2010_{1}_county10.zip".format(
-                base_url, self.fips
-            ),
-            "state": "{0}/STATE/2010/tl_2010_{1}_state10.zip".format(
-                base_url, self.fips
-            ),
-            "zcta": "{0}/ZCTA5/2010/tl_2010_{1}_zcta510.zip".format(
-                base_url, self.fips
-            ),
-            "block": "{0}/TABBLOCK/2010/tl_2010_{1}_tabblock10.zip".format(
-                base_url, self.fips
-            ),
-            "blockgroup": "{0}/BG/2010/tl_2010_{1}_bg10.zip".format(
-                base_url, self.fips
-            ),
+            "tract": urljoin(base, f"TRACT/2010/tl_2010_{fips}_tract10.zip"),
+            "cd": urljoin(base, f"CD/111/tl_2010_{fips}_cd111.zip"),
+            "county": urljoin(base, f"COUNTY/2010/tl_2010_{fips}_county10.zip"),
+            "state": urljoin(base, f"STATE/2010/tl_2010_{fips}_state10.zip"),
+            "zcta": urljoin(base, f"ZCTA5/2010/tl_2010_{fips}_zcta510.zip"),
+            "block": urljoin(base, f"TABBLOCK/2010/tl_2010_{fips}_tabblock10.zip"),
+            "blockgroup": urljoin(base, f"BG/2010/tl_2010_{fips}_bg10.zip"),
         }
 
-        if region and region in urls:
-            return urls[region]
-
         return urls
 
 
-def lookup(val, field=None, use_cache=True):
-    """ Semi-fuzzy state lookup. This method will make a best effort
-        attempt at finding the state based on the lookup value provided.
+def lookup(val, field: Optional[str] = None, use_cache: bool = True) -> Optional[State]:
+    """Semi-fuzzy state lookup. This method will make a best effort
+    attempt at finding the state based on the lookup value provided.
 
-          * two digits will search for FIPS code
-          * two letters will search for state abbreviation
-          * anything else will try to match the metaphone of state names
+      * two digits will search for FIPS code
+      * two letters will search for state abbreviation
+      * anything else will try to match the metaphone of state names
 
-        Metaphone is used to allow for incorrect, but phonetically accurate,
-        spelling of state names.
+    Metaphone is used to allow for incorrect, but phonetically accurate,
+    spelling of state names.
 
-        Exact matches can be done on any attribute on State objects by passing
-        the `field` argument. This skips the fuzzy-ish matching and does an
-        exact, case-sensitive comparison against the specified field.
+    Exact matches can be done on any attribute on State objects by passing
+    the `field` argument. This skips the fuzzy-ish matching and does an
+    exact, case-sensitive comparison against the specified field.
 
-        This method caches non-None results, but can the cache can be bypassed
-        with the `use_cache=False` argument.
+    This method caches non-None results, but can the cache can be bypassed
+    with the `use_cache=False` argument.
     """
 
+    matched_state = None
+
     if field is None:
         if FIPS_RE.match(val):
             field = "fips"
@@ -87,20 +95,23 @@ def lookup(val, field=None, use_cache=True):
             field = "name_metaphone"
 
     # see if result is in cache
-    cache_key = "%s:%s" % (field, val)
+    cache_key = f"{field}:{val}"
     if use_cache and cache_key in _lookup_cache:
-        return _lookup_cache[cache_key]
+        matched_state = _lookup_cache[cache_key]
 
-    for state in itertools.chain(STATES_AND_TERRITORIES, [DC], OBSOLETE):
+    for state in STATES_AND_TERRITORIES:
         if val == getattr(state, field):
-            _lookup_cache[cache_key] = state
-            return state
+            matched_state = state
+            if use_cache:
+                _lookup_cache[cache_key] = state
+
+    return matched_state
 
 
-def mapping(from_field, to_field, states=None):
+def mapping(from_field: str, to_field: str, states: Optional[Iterable[State]] = None) -> Dict[Any, Any]:
     if states is None:
-        states = itertools.chain(STATES_AND_TERRITORIES, [DC], OBSOLETE)
-    return dict((getattr(s, from_field), getattr(s, to_field)) for s in states)
+        states = STATES_AND_TERRITORIES
+    return {getattr(s, from_field): getattr(s, to_field) for s in states}
 
 
 AL = State(
@@ -1248,9 +1259,9 @@ WY = State(
 )
 
 
-OBSOLETE = [DK, OL, PI]
-TERRITORIES = [AS, GU, MP, PR, VI]
-STATES = [
+OBSOLETE: List[State] = [DK, OL, PI]
+TERRITORIES: List[State] = [AS, GU, MP, PR, VI]
+STATES: List[State] = [
     AL,
     AK,
     AZ,
@@ -1302,7 +1313,7 @@ STATES = [
     WI,
     WY,
 ]
-STATES_CONTIGUOUS = [
+STATES_CONTIGUOUS: List[State] = [
     AL,
     AZ,
     AR,
@@ -1352,7 +1363,7 @@ STATES_CONTIGUOUS = [
     WI,
     WY,
 ]
-STATES_CONTINENTAL = [
+STATES_CONTINENTAL: List[State] = [
     AL,
     AK,
     AZ,
@@ -1403,4 +1414,13 @@ STATES_CONTINENTAL = [
     WI,
     WY,
 ]
-STATES_AND_TERRITORIES = STATES + TERRITORIES
+
+STATES_AND_TERRITORIES: List[State] = STATES + TERRITORIES
+
+COMMONWEALTHS: List[State] = [KY, MA, PA, VA]
+
+if DC_STATEHOOD:
+    STATES.append(DC)
+    STATES_AND_TERRITORIES.append(DC)
+    STATES_CONTIGUOUS.append(DC)
+    STATES_CONTINENTAL.append(DC)
